@@ -29,6 +29,11 @@ from oslo_config import types
 load_tests = testscenarios.load_tests_apply_scenarios
 
 
+def custom_type(a):
+    """Something that acts like a type, but isn't known"""
+    return a
+
+
 class GeneratorTestCase(base.BaseTestCase):
 
     groups = {
@@ -80,9 +85,8 @@ class GeneratorTestCase(base.BaseTestCase):
                                   default='a',
                                   choices=(None, '', 'a', 'b', 'c'),
                                   help='a string with choices'),
-        'deprecated_opt': cfg.StrOpt('bar',
-                                     deprecated_name='foobar',
-                                     help='deprecated'),
+        'deprecated_opt_without_deprecated_group': cfg.StrOpt(
+            'bar', deprecated_name='foobar', help='deprecated'),
         'deprecated_for_removal_opt': cfg.StrOpt(
             'bar', deprecated_for_removal=True, help='deprecated for removal'),
         'deprecated_reason_opt': cfg.BoolOpt(
@@ -92,10 +96,13 @@ class GeneratorTestCase(base.BaseTestCase):
             deprecated_reason='This was supposed to work but it really, '
                               'really did not. Always buy house insurance.',
             help='DEPRECATED: Turn off stove'),
-        'deprecated_group': cfg.StrOpt('bar',
-                                       deprecated_group='group1',
-                                       deprecated_name='foobar',
-                                       help='deprecated'),
+        'deprecated_opt_with_deprecated_since': cfg.BoolOpt(
+            'tune_in',
+            deprecated_for_removal=True,
+            deprecated_since='13.0'),
+        'deprecated_opt_with_deprecated_group': cfg.StrOpt(
+            'bar', deprecated_name='foobar', deprecated_group='group1',
+            help='deprecated'),
         # Unknown Opt default must be a string
         'unknown_type': cfg.Opt('unknown_opt',
                                 default='123',
@@ -145,6 +152,9 @@ class GeneratorTestCase(base.BaseTestCase):
         'hostname_opt': cfg.HostnameOpt('hostname_opt',
                                         default='compute01.nova.site1',
                                         help='a hostname'),
+        'uri_opt': cfg.URIOpt('uri_opt',
+                              default='http://example.com',
+                              help='a URI'),
         'multi_opt': cfg.MultiStrOpt('multi_opt',
                                      default=['1', '2', '3'],
                                      help='multiple strings'),
@@ -160,9 +170,18 @@ class GeneratorTestCase(base.BaseTestCase):
         'string_type_with_bad_default': cfg.Opt('string_type_with_bad_default',
                                                 help='string with bad default',
                                                 default=4096),
+        'native_str_type': cfg.Opt('native_str_type',
+                                   help='native help',
+                                   type=str),
+        'native_int_type': cfg.Opt('native_int_type',
+                                   help='native help',
+                                   type=int),
+        'native_float_type': cfg.Opt('native_float_type',
+                                     help='native help',
+                                     type=float),
         'custom_type': cfg.Opt('custom_type',
                                help='custom help',
-                               type=type('string')),
+                               type=custom_type),
         'custom_type_name': cfg.Opt('custom_opt_type',
                                     type=types.Integer(type_name='port'
                                                        ' number'),
@@ -398,8 +417,10 @@ class GeneratorTestCase(base.BaseTestCase):
 # Allowed values: <None>, '', a, b, c
 #choices_opt = a
 ''')),
-        ('deprecated',
-         dict(opts=[('test', [(groups['foo'], [opts['deprecated_opt']])])],
+        ('deprecated opt without deprecated group',
+         dict(opts=[('test',
+                     [(groups['foo'],
+                       [opts['deprecated_opt_without_deprecated_group']])])],
               expected='''[DEFAULT]
 
 
@@ -411,7 +432,7 @@ class GeneratorTestCase(base.BaseTestCase):
 #
 
 # deprecated (string value)
-# Deprecated group/name - [DEFAULT]/foobar
+# Deprecated group/name - [foo]/foobar
 #bar = <None>
 ''')),
         ('deprecated_for_removal',
@@ -452,8 +473,10 @@ class GeneratorTestCase(base.BaseTestCase):
 # Always buy house insurance.
 #turn_off_stove = false
 ''')),
-        ('deprecated_group',
-         dict(opts=[('test', [(groups['foo'], [opts['deprecated_group']])])],
+        ('deprecated_opt_with_deprecated_group',
+         dict(opts=[('test',
+                     [(groups['foo'],
+                       [opts['deprecated_opt_with_deprecated_group']])])],
               expected='''[DEFAULT]
 
 
@@ -664,6 +687,39 @@ class GeneratorTestCase(base.BaseTestCase):
 
 # a string (string value)
 #str_opt = fooishbar
+''')),
+        ('native_str_type',
+         dict(opts=[('test', [(None, [opts['native_str_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (string value)
+#native_str_type = <None>
+''')),
+        ('native_int_type',
+         dict(opts=[('test', [(None, [opts['native_int_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (integer value)
+#native_int_type = <None>
+''')),
+        ('native_float_type',
+         dict(opts=[('test', [(None, [opts['native_float_type']])])],
+              expected='''[DEFAULT]
+
+#
+# From test
+#
+
+# native help (floating point value)
+#native_float_type = <None>
 ''')),
         ('multi_opt_sample_default',
          dict(opts=[('test', [(None, [opts['multi_opt_sample_default']])])],
@@ -1003,6 +1059,35 @@ class GeneratorAdditionalTestCase(base.BaseTestCase):
             actual = f.read()
         self.assertEqual(expected, actual)
 
+    def _test_output_default_list_opt_with_string_value(self, default):
+        opt = cfg.ListOpt('list_opt', help='a list', default=default)
+        config = [("namespace1", [
+                   ("alpha", [opt])])]
+        groups = generator._get_groups(config)
+
+        fd, tmp_file = tempfile.mkstemp()
+        f = open(tmp_file, 'w+')
+        formatter = generator._OptFormatter(output_file=f)
+        expected = '''[alpha]
+
+#
+# From namespace1
+#
+
+# a list (list value)
+#list_opt = %(default)s
+''' % {'default': default}
+        generator._output_opts(formatter, 'alpha', groups.pop('alpha'))
+        f.close()
+        content = open(tmp_file).read()
+        self.assertEqual(expected, content)
+
+    def test_output_default_list_opt_with_string_value_multiple_entries(self):
+        self._test_output_default_list_opt_with_string_value('foo,bar')
+
+    def test_output_default_list_opt_with_string_value_single_entry(self):
+        self._test_output_default_list_opt_with_string_value('foo')
+
 
 class GeneratorMutableOptionTestCase(base.BaseTestCase):
 
@@ -1010,7 +1095,7 @@ class GeneratorMutableOptionTestCase(base.BaseTestCase):
         out = moves.StringIO()
         opt = cfg.StrOpt('foo', help='foo option', mutable=True)
         gen = generator._OptFormatter(output_file=out)
-        gen.format(opt)
+        gen.format(opt, 'group1')
         result = out.getvalue()
         self.assertIn(
             'This option can be changed without restarting.',
@@ -1021,7 +1106,7 @@ class GeneratorMutableOptionTestCase(base.BaseTestCase):
         out = moves.StringIO()
         opt = cfg.StrOpt('foo', help='foo option', mutable=False)
         gen = generator._OptFormatter(output_file=out)
-        gen.format(opt)
+        gen.format(opt, 'group1')
         result = out.getvalue()
         self.assertNotIn(
             'This option can be changed without restarting.',
@@ -1099,4 +1184,39 @@ class ChangeDefaultsTestCase(base.BaseTestCase):
         self.assertIs(orig_opt, the_opt)
 
 
+class RequiredOptionTestCase(base.BaseTestCase):
+
+    opts = [cfg.StrOpt('foo', help='foo option', default='fred'),
+            cfg.StrOpt('bar', help='bar option', required=True),
+            cfg.StrOpt('foo_bar', help='foobar'),
+            cfg.StrOpt('bars', help='bars foo', required=True)]
+
+    def test_required_option_order_single_ns(self):
+
+        config = [("namespace1", [
+                   ("alpha", self.opts)])]
+        groups = generator._get_groups(config)
+
+        fd, tmp_file = tempfile.mkstemp()
+        with open(tmp_file, 'w+') as f:
+            formatter = generator._OptFormatter(output_file=f)
+            generator._output_opts(formatter, 'alpha',
+                                   groups.pop('alpha'), True)
+        expected = '''[alpha]
+
+#
+# From namespace1
+#
+
+# bar option (string value)
+bar = <None>
+
+# bars foo (string value)
+bars = <None>
+'''
+        with open(tmp_file, 'r') as f:
+            actual = f.read()
+        self.assertEqual(expected, actual)
+
+
 GeneratorTestCase.generate_scenarios()
